



if exist %LIBRARY_PREFIX%\\bin\\libcling.dll (exit 0) else (exit 1)
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %LIBRARY_PREFIX%\\lib\\libcling.lib (exit 0) else (exit 1)
IF %ERRORLEVEL% NEQ 0 exit /B 1
cling '2+3; 0'
IF %ERRORLEVEL% NEQ 0 exit /B 1
cling '#include <vector>'
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
